



from azure.search.documents import SearchClient
from azure.search.documents.models import VectorizableTextQuery

from azure.search.documents.models import (
    QueryType,
    QueryCaptionType,
    QueryAnswerType
)

import os
from azure.identity import DefaultAzureCredential
from azure.core.credentials import AzureKeyCredential

AZURE_SEARCH_SERVICE_ENDPOINT = os.getenv("AZURE_SEARCH_SERVICE_ENDPOINT")
AZURE_SEARCH_ADMIN_KEY = os.getenv("AZURE_SEARCH_ADMIN_KEY")

credential = AzureKeyCredential(AZURE_SEARCH_ADMIN_KEY) if len(AZURE_SEARCH_ADMIN_KEY) > 0 else DefaultAzureCredential()


def vector_query(query, index_name):

    # Pure Vector Search
    # query = "Which is more comprehensive, Northwind Health Plus vs Northwind Standard?"
    # index_name = "azureblob-index"
    search_client = SearchClient(AZURE_SEARCH_SERVICE_ENDPOINT, index_name, credential=credential)
    vector_query = VectorizableTextQuery(text=query, k_nearest_neighbors=1, fields="content", exhaustive=True)
    # vector_query = VectorizableTextQuery(text=query, k_nearest_neighbors=1, fields="vector", exhaustive=True)
    # Use the below query to pass in the raw vector query instead of the query vectorization
    # vector_query = RawVectorQuery(vector=generate_embeddings(query), k_nearest_neighbors=3, fields="vector")

    results = search_client.search(
        search_text=None,
        vector_queries= [vector_query],
        # select=["parent_id", "chunk_id", "chunk"],
        top=1
    )

    print(results)
    return results


# for result in results:
#   print(result)
    # print(f"parent_id: {result['parent_id']}")
    # print(f"chunk_id: {result['chunk_id']}")
    # print(f"Score: {result['@search.score']}")
    # print(f"Content: {result['chunk']}")



def hybrid_query(query, index_name):

    # Hybrid Search
    # query = "Which is more comprehensive, Northwind Health Plus vs Northwind Standard?"

    search_client = SearchClient(AZURE_SEARCH_SERVICE_ENDPOINT, index_name, credential=credential)
    vector_query = VectorizableTextQuery(text=query, k_nearest_neighbors=1, fields="vector", exhaustive=True)

    results = search_client.search(
        search_text=query,
        vector_queries= [vector_query],
        select=["parent_id", "chunk_id", "chunk"],
        top=1
    )

    return results

    # for result in results:
    #     print(f"parent_id: {result['parent_id']}")
    #     print(f"chunk_id: {result['chunk_id']}")
    #     print(f"Score: {result['@search.score']}")
    #     print(f"Content: {result['chunk']}")
    


def hybrid_semantic_query(query, index_name):
    
    # Semantic Hybrid Search
    # query = "Which is more comprehensive, Northwind Health Plus vs Northwind Standard?"

    search_client = SearchClient(AZURE_SEARCH_SERVICE_ENDPOINT, index_name, credential)
    vector_query = VectorizableTextQuery(text=query, k_nearest_neighbors=1, fields="vector", exhaustive=True)

    results = search_client.search(
        search_text=query,
        vector_queries=[vector_query],
        select=["parent_id", "chunk_id", "chunk"],
        query_type=QueryType.SEMANTIC,
        semantic_configuration_name='my-semantic-config',
        query_caption=QueryCaptionType.EXTRACTIVE,
        query_answer=QueryAnswerType.EXTRACTIVE,
        top=1
    )

    semantic_answers = results.get_answers()
    if semantic_answers:
        for answer in semantic_answers:
            if answer.highlights:
                print(f"Semantic Answer: {answer.highlights}")
            else:
                print(f"Semantic Answer: {answer.text}")
            print(f"Semantic Answer Score: {answer.score}\n")

    return results


    # for result in results:
    #     print(f"parent_id: {result['parent_id']}")
    #     print(f"chunk_id: {result['chunk_id']}")
    #     print(f"Reranker Score: {result['@search.reranker_score']}")
    #     print(f"Content: {result['chunk']}")

    #     captions = result["@search.captions"]
    #     if captions:
    #         caption = captions[0]
    #         if caption.highlights:
    #             print(f"Caption: {caption.highlights}\n")
    #         else:
    #             print(f"Caption: {caption.text}\n")




from azure.storage.filedatalake import DataLakeServiceClient
from azure.identity import DefaultAzureCredential
import os
import logging
import re

import asyncio

from azure.identity import ClientSecretCredential
from msgraph import GraphServiceClient

# Create a credential object. Used to authenticate requests
credential = ClientSecretCredential(
    tenant_id='803b8895-b793-4dc6-b891-6e822908030c',
    client_id='0253c6c2-a257-4bf0-af53-6c75f6d56527',
    client_secret='BnY8Q~~2uX5gKw1wHFvDsvb8sk1hyjLK3a6sJckv'
)
scopes = ['https://graph.microsoft.com/.default']

# Create an API client with the credentials and scopes.
client = GraphServiceClient(credentials=credential, scopes=scopes)


# from asgiref.sync import async_to_sync

# # dept_name = "GPTKB_HRTest"
# @async_to_sync
async def get_departments_group_ids():
    groups = await client.groups.get()
    depts_dict = {group.display_name: group.id for group in groups.value}
    return depts_dict





# import asyncio


# def get_departments_group_ids():
#     loop = asyncio.get_event_loop()
#     if loop.is_running():
#         groups = loop.run_until_complete(client.groups.get())
#     else:
#         new_loop = asyncio.new_event_loop()
#         asyncio.set_event_loop(new_loop)
#         groups = new_loop.run_until_complete(client.groups.get())
#         new_loop.close()
#     depts_dict = {group.display_name: group.id for group in groups.value}
#     return depts_dict




storage_account_name = os.getenv('STORAGE_ACCOUNT_NAME', "datalakechamcorag")
storage_account_key = os.getenv('STORAGE_ACCOUNT_KEY', "VUuFEeVBkXi1hHpIusMcGrWXtggm30WXgtOfQtXBDHEFepQ2zA+mVlewgNU7s60YVbgnnp9vykiZ+AStj5mmIg==")
# blob_container_name = os.getenv('BLOB_CONTAINER_NAME', "gptkbcontainer")

# set up the service client with the credentials from the environment variables
service_client = DataLakeServiceClient(account_url="{}://{}.dfs.core.windows.net".format(
    "https",
    storage_account_name
), credential=storage_account_key)


def get_clients(blob_container_name, dept_name, file_name):
    filesystem_client = service_client.get_file_system_client(blob_container_name)
    directory_client = filesystem_client.get_directory_client(dept_name)
    file_client = directory_client.get_file_client(file_name)
    return file_client, directory_client


def get_updated_file_metadata(file_metadata, department_group_id):

    updated_file_metadata = file_metadata.copy()

    if 'group_ids' in file_metadata:
        existing_group_ids = file_metadata['group_ids']
        if department_group_id not in existing_group_ids:
            updated_file_metadata['group_ids'] += f", {department_group_id}"
    else:
        updated_file_metadata['group_ids'] = str(department_group_id)
    
    return updated_file_metadata


def update_file_metadata(file_name, dept_name, blob_container_name):
    logging.info(f"[Ok]  Getting  directory client for {dept_name}")
    logging.info(f"[Ok]  Getting  file client for {file_name}")

    file_client, directory_client = get_clients(blob_container_name, dept_name, file_name)

    directory_info = directory_client.get_access_control()
    logging.info(f"[Ok]  Getting  directory_info {directory_info}")
    # file_info = file_client.get_access_control()
    acl = directory_info.get('acl')
    owner_group = directory_info.get('group')
    other_groups = re.findall(r'group:([\w-]+)', acl)

    logging.info(f"[Ok]  owner_group  {owner_group}")
    logging.info(f"[Ok]  other_groups {other_groups}")

    other_groups.append(owner_group) 
    group_ids = ', '.join(other_groups)
    
    file_metadata = file_client.get_file_properties()['metadata']
    updated_file_metadata = get_updated_file_metadata(file_metadata, group_ids)

    logging.info(f"[Ok]  current file metadata {file_metadata}")
    file_client.set_metadata(updated_file_metadata)
    logging.info(f"[Ok]  file metadata update with groups ids {updated_file_metadata}")







from azure.search.documents.indexes import SearchIndexerClient
from azure.search.documents.indexes.models import (
    SearchIndexerDataContainer,
    SearchIndexerDataSourceConnection
)
from azure.identity import DefaultAzureCredential
from azure.core.credentials import AzureKeyCredential
from azure.search.documents.indexes._generated.models import NativeBlobSoftDeleteDeletionDetectionPolicy
import os

import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')



# AZURE_SEARCH_INDEX = os.getenv("AZURE_SEARCH_INDEX")
BLOB_CONNECTION_STRING = os.getenv("BLOB_CONNECTION_STRING")
BLOB_CONTAINER_NAME = os.getenv("BLOB_CONTAINER_NAME")

AZURE_SEARCH_SERVICE_ENDPOINT = os.getenv("AZURE_SEARCH_SERVICE_ENDPOINT")
AZURE_SEARCH_ADMIN_KEY = os.getenv("AZURE_SEARCH_ADMIN_KEY")

credential = AzureKeyCredential(AZURE_SEARCH_ADMIN_KEY) if len(AZURE_SEARCH_ADMIN_KEY) > 0 else DefaultAzureCredential()


def create_datasource(index_name, blob_container_name=BLOB_CONTAINER_NAME, blob_connection_string=BLOB_CONNECTION_STRING):
    
    logging.info(f"[Ok] index_name name {index_name}")

    # Create a data source
    indexer_client = SearchIndexerClient(AZURE_SEARCH_SERVICE_ENDPOINT, credential)
    container = SearchIndexerDataContainer(name=blob_container_name)
    # container = SearchIndexerDataContainer(name=blob_container_name, query=blob_folder_name)
    data_source_connection = SearchIndexerDataSourceConnection(
        name=f"{index_name}-datasource",
        type="adlsgen2",
        connection_string=blob_connection_string,
        container=container,
        # data_deletion_detection_policy=NativeBlobSoftDeleteDeletionDetectionPolicy()
    )
    data_source = indexer_client.create_or_update_data_source_connection(data_source_connection)
    
    logging.info(f"[Ok] data_source created or updated")

    return data_source





from azure.search.documents.indexes.models import (
    SplitSkill,
    InputFieldMappingEntry,
    OutputFieldMappingEntry,
    AzureOpenAIEmbeddingSkill,
    SearchIndexerIndexProjections,
    SearchIndexerIndexProjectionSelector,
    SearchIndexerIndexProjectionsParameters,
    IndexProjectionMode,
    SearchIndexerSkillset
)
from azure.search.documents.indexes import SearchIndexerClient
from azure.search.documents.indexes.models import (
    SearchIndexerDataContainer,
    SearchIndexerDataSourceConnection
)
from azure.search.documents.indexes._generated.models import NativeBlobSoftDeleteDeletionDetectionPolicy

from azure.core.credentials import AzureKeyCredential
import os
from azure.identity import DefaultAzureCredential


from azure.search.documents.indexes.models import (
    SearchIndexer,
    FieldMapping
)

import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')


BLOB_CONNECTION_STRING = os.getenv("BLOB_CONNECTION_STRING")
BLOB_CONTAINER_NAME = os.getenv("BLOB_CONTAINER_NAME")
AZURE_OPENAI_EMBEDDING_DEPLOYMENT = os.getenv("AZURE_OPENAI_EMBEDDING_DEPLOYMENT")


AZURE_SEARCH_SERVICE_ENDPOINT = os.getenv("AZURE_SEARCH_SERVICE_ENDPOINT")
AZURE_SEARCH_ADMIN_KEY = os.getenv("AZURE_SEARCH_ADMIN_KEY")
AZURE_OPENAI_KEY = os.getenv("AZURE_OPENAI_KEY")
AZURE_OPENAI_MODEL_DEPLOYMENT = os.getenv("AZURE_OPENAI_MODEL_DEPLOYMENT")
AZURE_OPENAI_ENDPOINT = os.getenv("AZURE_OPENAI_ENDPOINT")


credential = AzureKeyCredential(AZURE_SEARCH_ADMIN_KEY) if len(AZURE_SEARCH_ADMIN_KEY) > 0 else DefaultAzureCredential()
indexer_client = SearchIndexerClient(AZURE_SEARCH_SERVICE_ENDPOINT, credential)


def create_indexer(index_name, data_source, skillset_name):

    # Create an indexer
    indexer_name = f"{index_name}-indexer"

    indexer = SearchIndexer(
        name=indexer_name,
        description="Indexer to index documents and generate embeddings",
        skillset_name=skillset_name,
        target_index_name=index_name,
        data_source_name=data_source.name,
        # Map the metadata_storage_name field to the title field in the index to display the PDF title in the search results
        field_mappings=[FieldMapping(source_field_name="metadata_storage_name", target_field_name="title")]
    )

    indexer_result = indexer_client.create_or_update_indexer(indexer)
    logging.info(f"[Ok] {indexer_result.name} indexer update completed")

    return indexer_result



import os
import asyncio

from dotenv import load_dotenv
load_dotenv()

import sys
# sys.path.append('/home/azureuser/projects/rag_chatbot/src/chamco_ragbot')


import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

from chamco_ragbot.utils import parse_file_url, sanitize_name, parse_sharepoint_link

from chamco_ragbot.filetransfer import sharepoint_auth, download_sharepoint_file, upload_file_to_blob_container
from chamco_ragbot.acl import get_departments_group_ids, update_file_metadata
from chamco_ragbot.dept import process_departments

from chamco_ragbot.indexer import create_indexer, indexer_client

# from .filetransfer import sharepoint_auth, download_sharepoint_file, upload_file_to_blob_container
from chamco_ragbot.datasource import create_datasource
# from .chat import chat, get_context, get_response
from chamco_ragbot.index import create_index
from chamco_ragbot.indexer import create_indexer, indexer_client
from chamco_ragbot.skillset import create_skill_set

SHAREPOINT_SITE_URL = os.getenv("SHAREPOINT_SITE_URL")
SHAREPOINT_USERNAME = os.getenv("SHAREPOINT_USERNAME")
SHAREPOINT_PASSWORD = os.getenv("SHAREPOINT_PASSWORD")

BLOB_CONNECTION_STRING = os.getenv("BLOB_CONNECTION_STRING")
BLOB_CONTAINER_NAME = os.getenv("BLOB_CONTAINER_NAME")



def update_blob_container(item_link):
    # folder_full, folder_name, file_name = parse_file_url(file_url)
    folder_name, file_name, file_url, site_link, full_folder_name = parse_sharepoint_link(item_link)
    dept_name = folder_name
    blob_name = os.path.join(folder_name, file_name)
    ctx = sharepoint_auth(site_link, SHAREPOINT_USERNAME, SHAREPOINT_PASSWORD)
    download_path = download_sharepoint_file(ctx, file_url)
    blob_name = upload_file_to_blob_container(download_path, blob_name, BLOB_CONNECTION_STRING, BLOB_CONTAINER_NAME)
        
    # Move subsequent code here
    update_file_metadata(file_name, dept_name, BLOB_CONTAINER_NAME)



def update_aisearch():

    folder_name = sanitize_name(BLOB_CONTAINER_NAME)
    logging.info(f"[Ok] {folder_name} folder name sanitized")
    logging.info(f"[Ok] {BLOB_CONTAINER_NAME} BLOB_CONTAINER_NAME")
    index = create_index(BLOB_CONTAINER_NAME)
    logging.info(f"[Ok] {index.name} index name")
    data_source = create_datasource(index_name=index.name)
    skillset = create_skill_set(index_name=index.name)
    # indexer_client.close()
    indexer_result = create_indexer(index.name, data_source, skillset.name)

    logging.info(f"[Ok] {indexer_result.name} indexer update completed")
    logging.info(f"[Ok] Resetting indexer {indexer_result.name}")
    indexer_client.reset_indexer(indexer_result.name)
    logging.info(f"[Ok] running indexer {indexer_result.name}")
    indexer_client.run_indexer(indexer_result.name)
    logging.info(f"[Ok] running indexer {indexer_result.name} completed")
    logging.info(f"[Ok] rag update completed")




from azure.search.documents.indexes.models import (
    SplitSkill,
    InputFieldMappingEntry,
    OutputFieldMappingEntry,
    AzureOpenAIEmbeddingSkill,
    SearchIndexerIndexProjections,
    SearchIndexerIndexProjectionSelector,
    SearchIndexerIndexProjectionsParameters,
    IndexProjectionMode,
    SearchIndexerSkillset
)
from azure.search.documents.indexes import SearchIndexerClient
import os
from azure.core.credentials import AzureKeyCredential
from azure.identity import DefaultAzureCredential

import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')



AZURE_SEARCH_INDEX = os.getenv("AZURE_SEARCH_INDEX")
# BLOB_CONNECTION_STRING = os.getenv("BLOB_CONNECTION_STRING")
# BLOB_CONTAINER_NAME = os.getenv("BLOB_CONTAINER_NAME")


AZURE_SEARCH_SERVICE_ENDPOINT = os.getenv("AZURE_SEARCH_SERVICE_ENDPOINT")
AZURE_SEARCH_ADMIN_KEY = os.getenv("AZURE_SEARCH_ADMIN_KEY")
AZURE_OPENAI_KEY = os.getenv("AZURE_OPENAI_KEY")
# # AZURE_OPENAI_MODEL_DEPLOYMENT = os.getenv("AZURE_OPENAI_MODEL_DEPLOYMENT")
AZURE_OPENAI_ENDPOINT = os.getenv("AZURE_OPENAI_ENDPOINT")
AZURE_OPENAI_EMBEDDING_DEPLOYMENT = os.getenv("AZURE_OPENAI_EMBEDDING_DEPLOYMENT")


credential = AzureKeyCredential(AZURE_SEARCH_ADMIN_KEY) if len(AZURE_SEARCH_ADMIN_KEY) > 0 else DefaultAzureCredential()


def create_skill_set(index_name):
    # Create a skillset
    skillset_name = f"{index_name}-skillset"

    split_skill = SplitSkill(
        description="Split skill to chunk documents",
        text_split_mode="pages",
        context="/document",
        maximum_page_length=2000,
        page_overlap_length=500,
        inputs=[
            InputFieldMappingEntry(name="text", source="/document/content"),
        ],
        outputs=[
            OutputFieldMappingEntry(name="textItems", target_name="pages")
        ],
    )

    embedding_skill = AzureOpenAIEmbeddingSkill(
        description="Skill to generate embeddings via Azure OpenAI",
        context="/document/pages/*",
        resource_uri=AZURE_OPENAI_ENDPOINT,
        deployment_id=AZURE_OPENAI_EMBEDDING_DEPLOYMENT,
        api_key=AZURE_OPENAI_KEY,
        inputs=[
            InputFieldMappingEntry(name="text", source="/document/pages/*"),
        ],
        outputs=[
            OutputFieldMappingEntry(name="embedding", target_name="vector")
        ],
    )

    index_projections = SearchIndexerIndexProjections(
        selectors=[
            SearchIndexerIndexProjectionSelector(
                target_index_name=index_name,
                parent_key_field_name="parent_id",
                source_context="/document/pages/*",
                mappings=[
                    InputFieldMappingEntry(name="chunk", source="/document/pages/*"),
                    InputFieldMappingEntry(name="vector", source="/document/pages/*/vector"),
                    InputFieldMappingEntry(name="title", source="/document/metadata_storage_name"),
                ],
            ),
        ],
        parameters=SearchIndexerIndexProjectionsParameters(
            projection_mode=IndexProjectionMode.SKIP_INDEXING_PARENT_DOCUMENTS
        ),
    )

    skillset = SearchIndexerSkillset(
        name=skillset_name,
        description="Skillset to chunk documents and generating embeddings",
        # skills=[split_skill, embedding_skill],
        skills=[split_skill],
        # index_projections=index_projections,
    )

    client = SearchIndexerClient(AZURE_SEARCH_SERVICE_ENDPOINT, credential)
    client.create_or_update_skillset(skillset)

    logging.info(f"[Ok] {skillset.name} skillset update completed")

    print(f"{skillset.name} created")
    return skillset




from azure.search.documents.indexes import SearchIndexClient
from azure.search.documents.indexes.models import (
    SearchField,
    SearchableField,
    SearchFieldDataType,
    VectorSearch,
    HnswAlgorithmConfiguration,
    HnswParameters,
    VectorSearchAlgorithmMetric,
    ExhaustiveKnnAlgorithmConfiguration,
    ExhaustiveKnnParameters,
    VectorSearchProfile,
    AzureOpenAIVectorizer,
    AzureOpenAIParameters,
    SemanticConfiguration,
    SemanticSearch,
    SemanticPrioritizedFields,
    SemanticField,
    SearchIndex
)

from azure.search.documents.indexes.models import (
    SearchIndexer,
    FieldMapping
)
from azure.identity import DefaultAzureCredential
from azure.core.credentials import AzureKeyCredential
import os

import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')


AZURE_OPENAI_EMBEDDING_DEPLOYMENT = os.getenv("AZURE_OPENAI_EMBEDDING_DEPLOYMENT")
AZURE_SEARCH_SERVICE_ENDPOINT = os.getenv("AZURE_SEARCH_SERVICE_ENDPOINT")
AZURE_SEARCH_ADMIN_KEY = os.getenv("AZURE_SEARCH_ADMIN_KEY")
AZURE_OPENAI_KEY = os.getenv("AZURE_OPENAI_KEY")
AZURE_OPENAI_MODEL_DEPLOYMENT = os.getenv("AZURE_OPENAI_MODEL_DEPLOYMENT")
AZURE_OPENAI_ENDPOINT = os.getenv("AZURE_OPENAI_ENDPOINT")

credential = AzureKeyCredential(AZURE_SEARCH_ADMIN_KEY) if len(AZURE_SEARCH_ADMIN_KEY) > 0 else DefaultAzureCredential()

# Create a search index
index_client = SearchIndexClient(endpoint=AZURE_SEARCH_SERVICE_ENDPOINT, credential=credential)
# FIELDS = [
#     SearchField(name="parent_id", type=SearchFieldDataType.String, sortable=True, filterable=True, facetable=True),
#     SearchField(name="title", type=SearchFieldDataType.String),
#     SearchField(name="chunk_id", type=SearchFieldDataType.String, key=True, sortable=True, filterable=True, facetable=True, analyzer_name="keyword"),
#     SearchField(name="chunk", type=SearchFieldDataType.String, sortable=False, filterable=False, facetable=False),
#     SearchField(name="vector", type=SearchFieldDataType.Collection(SearchFieldDataType.Single), vector_search_dimensions=1536, vector_search_profile_name="myHnswProfile"),
# ]

FIELDS=[
            SearchField(name="parent_id", type=SearchFieldDataType.String, sortable=True, filterable=True, facetable=True),
            SearchField(name="title", type=SearchFieldDataType.String),
            SearchField(name="chunk_id", type=SearchFieldDataType.String, key=True, sortable=True, filterable=True, facetable=True, analyzer_name="keyword"),
            SearchField(name="chunk", type=SearchFieldDataType.String, sortable=False, filterable=False, facetable=False),
            # SearchField(name="vector", type=SearchFieldDataType.Collection(SearchFieldDataType.Single), vector_search_dimensions=1536, vector_search_profile_name="myHnswProfile"),
            # SearchField(name="id", type="Edm.String", key=True, searchable=False),
            # SearchField(name="content", type="Edm.String", searchable=True, filterable=False, sortable=False),
            SearchableField(name="content", type="Edm.String", analyzer_name="en.lucene"),
            # SearchField(name="metadata_storage_name", type="Edm.String", searchable=False, filterable=True, sortable=True, retrievable=True),
            # SearchField(name="metadata_storage_size", type="Edm.Int64", searchable=False, filterable=True, sortable=True),
            # SearchField(name="metadata_storage_content_type", type="Edm.String", searchable=False, filterable=True, sortable=True),
            SearchField(name="group_ids", type="Collection(Edm.String)", searchable=True, filterable=True, retrievable=True),
            # SearchField(name="group_ids", type="Edm.String", searchable=True, filterable=True)
    ]


def create_index(index_name):

    # # Create a search index
    # index_client = SearchIndexClient(endpoint=endpoint, credential=credential)
    # fields = [
    #     SearchField(name="parent_id", type=SearchFieldDataType.String, sortable=True, filterable=True, facetable=True),
    #     SearchField(name="title", type=SearchFieldDataType.String),
    #     SearchField(name="chunk_id", type=SearchFieldDataType.String, key=True, sortable=True, filterable=True, facetable=True, analyzer_name="keyword"),
    #     SearchField(name="chunk", type=SearchFieldDataType.String, sortable=False, filterable=False, facetable=False),
    #     SearchField(name="vector", type=SearchFieldDataType.Collection(SearchFieldDataType.Single), vector_search_dimensions=1536, vector_search_profile_name="myHnswProfile"),
    # ]

    # index_name = file_url.split('/')[-2].lower()

    # Configure the vector search configuration
    vector_search = VectorSearch(
        algorithms=[
            HnswAlgorithmConfiguration(
                name="myHnsw",
                parameters=HnswParameters(
                    m=4,
                    ef_construction=400,
                    ef_search=500,
                    metric=VectorSearchAlgorithmMetric.COSINE,
                ),
            ),
            ExhaustiveKnnAlgorithmConfiguration(
                name="myExhaustiveKnn",
                parameters=ExhaustiveKnnParameters(
                    metric=VectorSearchAlgorithmMetric.COSINE,
                ),
            ),
        ],
        profiles=[
            VectorSearchProfile(
                name="myHnswProfile",
                algorithm_configuration_name="myHnsw",
                vectorizer="myOpenAI",
            ),
            VectorSearchProfile(
                name="myExhaustiveKnnProfile",
                algorithm_configuration_name="myExhaustiveKnn",
                vectorizer="myOpenAI",
            ),
        ],
        vectorizers=[
            AzureOpenAIVectorizer(
                name="myOpenAI",
                kind="azureOpenAI",
                azure_open_ai_parameters=AzureOpenAIParameters(
                    resource_uri=AZURE_OPENAI_ENDPOINT,
                    deployment_id=AZURE_OPENAI_EMBEDDING_DEPLOYMENT,
                    api_key=AZURE_OPENAI_KEY,
                ),
            ),
        ],
    )

    semantic_config = SemanticConfiguration(
        name="my-semantic-config",
        prioritized_fields=SemanticPrioritizedFields(
            content_fields=[SemanticField(field_name="chunk")]
        ),
    )

    # Create the semantic search with the configuration
    semantic_search = SemanticSearch(configurations=[semantic_config])

    # Create the search index
    # index = SearchIndex(name=index_name, fields=FIELDS, vector_search=vector_search, semantic_search=semantic_search)
    index = SearchIndex(name=index_name, fields=FIELDS)
    logging.info(f"[Ok] {index_name} created")

    result = index_client.create_or_update_index(index)
    logging.info(f"[Ok] index client created or updated")

    return result








def parse_file_url(file_url):
    # Split the path into folder and file parts
    parts = file_url.split('/')
    
    # Extract the folder name (without file name)
    folder_full = '/'.join(parts[:-1])
    folder_name = parts[-2]
    
    # Extract the file name
    file_name = parts[-1]
    
    return folder_full, folder_name, file_name




import re
from urllib.parse import unquote

def sanitize_name(folder_name):
    # Convert to lowercase
    folder_name = folder_name.lower()
    
    # Remove non-alphanumeric characters and replace spaces with dashes
    folder_name = re.sub(r'[^a-z0-9\s-]', '', folder_name)
    
    # Replace multiple spaces with a single dash
    folder_name = re.sub(r'\s+', '-', folder_name)
    
    # Remove leading and trailing dashes
    folder_name = folder_name.strip('-')
    
    # Limit to 128 characters
    folder_name = folder_name[:128]
    
    return folder_name



def parse_sharepoint_link(item_link):
    # Define a regex pattern to match the SharePoint site URL
    site_pattern = r'https://[^/]+/sites/[^/]+'
    
    # Extract the site link using regex
    site_match = re.match(site_pattern, item_link)
    site_link = site_match.group() if site_match else None
    
    # Extract the file URL by removing the site link part
    file_url = item_link[len(site_link)+1:] if site_link else None
    
    # Unquote the file URL to remove %20 encoding
    file_url = unquote(file_url) if file_url else None
    
    # Split the file URL to get the folder name and file name
    parts = file_url.split('/')
    folder_name = parts[-2]
    full_folder_name = '/'.join(parts[:-1]) if len(parts) > 1 else None
    file_name = parts[-1] if len(parts) > 0 else None
    
    return folder_name, file_name, file_url, site_link, full_folder_name


"""
Module for setting up the project.
"""

from setuptools import find_packages, setup

setup(
    name="chamco_ragbot",
    version="0.1.39",
    packages=find_packages(exclude=["tests*"]),
    license="MIT",
    description=" ",
    author="Ibrahim Animashaun",
)


import logging

# from chamco_ragbot.utils import parse_file_url, sanitize_folder_name
from chamco_ragbot.acl import update_file_metadata



def process_departments(depts_dict, dept_name, file_name, BLOB_CONTAINER_NAME):
    # folder_full, folder_name, file_name = parse_file_url(file_url)

    if depts_dict is not None:
        department_group_id = depts_dict.get(dept_name)
        if department_group_id is not None:
            update_file_metadata(file_name, dept_name, department_group_id, BLOB_CONTAINER_NAME)
            
            
            logging.info("[Ok] RAG update completed")
        else:
            logging.error(f"Department group ID not found for {dept_name}.")
    else:
        logging.error("Error: Departments dictionary not populated.")




def process_departments_new(dept_name, file_name, BLOB_CONTAINER_NAME):
    # folder_full, folder_name, file_name = parse_file_url(file_url)

    # if dept_name is not None:
    #     department_group_id = depts_dict.get(dept_name)
    #     if department_group_id is not None:
    update_file_metadata(file_name, dept_name, BLOB_CONTAINER_NAME)
            
            
    #         logging.info("[Ok] RAG update completed")
    #     else:
    #         logging.error(f"Department group ID not found for {dept_name}.")
    # else:
    #     logging.error("Error: Departments dictionary not populated.")




import os
from typing import Dict
from azure.storage.blob import BlobServiceClient
from urllib.parse import urlparse, unquote_plus
from office365.sharepoint.client_context import ClientContext
from requests_ntlm import HttpNtlmAuth
import requests, tempfile

from office365.runtime.auth.user_credential import UserCredential
from office365.sharepoint.client_context import ClientContext

from azure.storage.blob import BlobClient
import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')


SHAREPOINT_SITE_URL = os.getenv("SHAREPOINT_SITE_URL")
SHAREPOINT_USERNAME = os.getenv("SHAREPOINT_USERNAME")
SHAREPOINT_PASSWORD = os.getenv("SHAREPOINT_PASSWORD")
BLOB_CONNECTION_STRING = os.getenv("BLOB_CONNECTION_STRING")
BLOB_CONTAINER_NAME = os.getenv("BLOB_CONTAINER_NAME")


def sharepoint_auth(SHAREPOINT_SITE_URL, SHAREPOINT_USERNAME, SHAREPOINT_PASSWORD):
    ctx = ClientContext(SHAREPOINT_SITE_URL).with_credentials(UserCredential(f"{SHAREPOINT_USERNAME}", f"{SHAREPOINT_PASSWORD}"))
    return ctx



def download_sharepoint_file(ctx, file_url):
        
    download_path = os.path.join(tempfile.mkdtemp(), os.path.basename(file_url))
    with open(download_path, "wb") as local_file:
        file = (
            ctx.web.get_file_by_server_relative_path(file_url)
            .download(local_file)
            .execute_query()
        )
        logging.info("[Ok] file has been downloaded into: {0}".format(download_path))
        # print("[Ok] file has been downloaded into: {0}".format(download_path))
    return download_path



def upload_file_to_blob_container(download_path, blob_name, BLOB_CONNECTION_STRING, BLOB_CONTAINER_NAME):

    blob = BlobClient.from_connection_string(conn_str=BLOB_CONNECTION_STRING, container_name=BLOB_CONTAINER_NAME, blob_name=blob_name)

    with open(download_path, "rb") as data:
        blob.upload_blob(data, overwrite=True)

        logging.info("[Ok] file has been uploaded to: {0}".format(blob_name))
        # print("[Ok] file has been uploaded to: {0}".format(blob_name))
    return blob_name





import os
import json
import openai
from dotenv import load_dotenv
load_dotenv()


# AZURE_SEARCH_INDEX = os.getenv("AZURE_SEARCH_INDEX")
# BLOB_CONNECTION_STRING = os.getenv("BLOB_CONNECTION_STRING")
# BLOB_CONTAINER_NAME = os.getenv("BLOB_CONTAINER_NAME")
# AZURE_OPENAI_EMBEDDING_DEPLOYMENT = os.getenv("AZURE_OPENAI_EMBEDDING_DEPLOYMENT")


AZURE_SEARCH_SERVICE_ENDPOINT = os.getenv("AZURE_SEARCH_SERVICE_ENDPOINT")
AZURE_SEARCH_ADMIN_KEY = os.getenv("AZURE_SEARCH_ADMIN_KEY")
AZURE_OPENAI_KEY = os.getenv("AZURE_OPENAI_KEY")
AZURE_OPENAI_MODEL_DEPLOYMENT = os.getenv("AZURE_OPENAI_MODEL_DEPLOYMENT")
AZURE_OPENAI_ENDPOINT = os.getenv("AZURE_OPENAI_ENDPOINT")


client = openai.AzureOpenAI(
    base_url=f"{AZURE_OPENAI_ENDPOINT}/openai/deployments/{AZURE_OPENAI_MODEL_DEPLOYMENT}/extensions",
    api_key=AZURE_OPENAI_KEY,
    api_version="2023-09-01-preview"
)


def chat(query, search_index):
    completion = client.chat.completions.create(
        messages=[{"role": "user", "content": query}],
        model=AZURE_OPENAI_MODEL_DEPLOYMENT,
        extra_body={
            "dataSources": [
                {
                    "type": "AzureCognitiveSearch",
                    "parameters": {
                        "endpoint": AZURE_SEARCH_SERVICE_ENDPOINT,
                        "key": AZURE_SEARCH_ADMIN_KEY,
                        "indexName": search_index,
                    }
                }
            ]
        }
    )

    return completion



def get_response(completion):

    role = completion.choices[0].message.role
    response = completion.choices[0].message.content

    return {role: response}

def get_context(completion):
    context = completion.choices[0].message.model_extra['context']['messages'][0]['content']
    parsed_context = parse_context(context)
    return parsed_context


def parse_context(context):
    json_context = json.loads(context)
    citations = json_context['citations']
    intent = json.loads(json_context['intent'])
    parsed_context = {"citations": citations, "intent": intent}
    return parsed_context